/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_LOADING

#include "util/loading/load_param.h"
#include "load_param_class_description.h"
#include "compiler.h"
#include "debug.h"
/**
 * @brief Constructs a new LoadParameter
 * @param root the XML-element to load this Parameter from
 * @param paramName the Parameter to load
 * @param object the BaseObject, to load this parameter on to (will be cast to executor's Parameter)
 * @param inLoadCycle If we are in a LoadCycle (loading differs.).
 */
LoadParamBase::LoadParamBase(const TiXmlElement* root, const std::string& paramName, BaseObject* object, bool inLoadCycle)
    :  object(object), paramName(paramName), inLoadCycle(inLoadCycle)
{
  // determin the LoadString.
  if (likely(!inLoadCycle))
    this->loadElem = grabParameterElement(root, paramName);
  else if (paramName == root->Value())
    this->loadElem = (TiXmlElement*)root->FirstChild();
  else
    this->loadElem = NULL;
}


/**
 * @param descriptionText The text to set as a description for this Parameter
 * @returns a pointer to itself.
 */
void LoadParamBase::describe(const std::string& descriptionText)
{
  /// TODO REIMPLEMENT
  /*  if (LoadParamClassDescription::parametersDescription && this->paramDesc && this->paramDesc->getDescription().empty())
  {
    this->paramDesc->setDescription(descriptionText);
  }*/
}



/**
 * @brief generates a LoadParam based on
 * @param root the Root Element to load onto the object.
 * @param paramName the Parameter name that is loaded.
 * @param object the Object to apply the changes on.
 * @param executor the Functional Object, that actually executes the function Call.
 * @param inLoadCycle If we are inside of a loading cycle. (Loading will be different here)
 */
CLoadParam::CLoadParam(const TiXmlElement* root, const std::string& paramName, BaseObject* object, Executor<const SubString>* executor, bool inLoadCycle)
    : LoadParamBase(root, paramName, object, inLoadCycle)
{
  assert (executor != NULL);
  // set the Executor.
  this->executor = executor;
  //if (this->executor)
  //  this->executor->setName(paramName);
}

/**
 * This is a VERY SPECIAL deconsrtuctor.
 * It is made, so that it loads the Parameters on destruction.
 * meaning, if an Executor a valid Object exist, and all
 * Execution-Conditions are met, they are executed here.
 */
CLoadParam::~CLoadParam()
{
  assert(this->object != NULL);

  std::string loadString;
  if (this->loadElem != NULL &&  this->loadElem->ToText())
  {
    loadString = this->loadElem->Value();
    if (!loadString.empty())
    {
      PRINTF(4)("Loading value '%s' with Parameters '%s' onto: %s::%s\n",
                this->paramName.c_str(), loadString.c_str(), this->object->getClassCName(), this->object->getCName());
      (*this->executor)(this->object, SubString(loadString, ",", SubString::WhiteSpaces, false, '\\'));
    }
  }
  delete this->executor;
}

/**
 * @brief set the default values of the executor
 * @param value0 the first default value
 * @param value1 the second default value
 * @param value2 the third default value
 * @param value3 the fourth default value
 * @param value4 the fifth default value
 */
CLoadParam& CLoadParam::defaultValues(const MultiType& value0, const MultiType& value1,
                                      const MultiType& value2, const MultiType& value3,
                                      const MultiType& value4)
{
  assert(this->executor != NULL);
  this->executor->defaultValues(value0, value1, value2, value3, value4);

  return *this;
}


//////////////////////
// HELPER FUNCTIONS //
//////////////////////
/**
 * @param root: The XML-element to grab a parameter from
 * @param parameterName: the parameter to grab
 * @returns the Value of the parameter if found, NULL otherwise
*/
std::string grabParameter(const TiXmlElement* root, const std::string& parameterName)
{
  const TiXmlElement* element;
  const TiXmlNode* node;

  if (root == NULL)
    return "";

  element = root->FirstChildElement( parameterName);
  if( element == NULL) return "";

  node = element->FirstChild();
  while( node != NULL)
  {
    if( node->ToText()) return node->Value();
    node = node->NextSibling();
  }
  return "";
}

/**
 * @param root: The XML-element to grab a parameter from
 * @param parameterName: the parameter to grab
 * @returns the Element of the parameter if found, NULL otherwise
 */
const TiXmlElement* grabParameterElement(const TiXmlElement* root, const std::string& parameterName)
{
  const TiXmlElement* element;
  const TiXmlNode* node;

  if (root == NULL)
    return NULL;

  element = root->FirstChildElement( parameterName);
  if( element == NULL) return NULL;

  node = element->FirstChild();
  while( node != NULL)
  {
    if( node->ToText()) return (TiXmlElement*)node;
    node = node->NextSibling();
  }
  return NULL;
}



